Module calc

Module calc 

Slot calculator for determining the current slot and timepoint within a slot.

Structs

SlotCalculator
A slot calculator, which can calculate slot numbers, windows, and offsets for a given chain.

Enums

SlotCalculatorEnvError
Generated error type for FromEnv for SlotCalculator . This error type is used to represent errors that occur when trying to create an instance of the struct from environment variables.