pub enum FromEnvErr<Inner> {
EnvError(String, VarError),
Empty(String),
ParseError(Inner),
}Expand description
Error type for loading from the environment. See the FromEnv trait for
more information.
Variants
EnvError(String, VarError)
The environment variable is missing.
Empty(String)
The environment variable is empty.
ParseError(Inner)
The environment variable is present, but the value could not be parsed.
Implementations
Sourceimpl FromEnvErr<Infallible>
impl FromEnvErr<Infallible>
Sourcepub fn infallible_into<T>(self) -> FromEnvErr<T>
pub fn infallible_into<T>(self) -> FromEnvErr<T>
Convert the error into another error type.
Sourceimpl<Inner> FromEnvErr<Inner>
impl<Inner> FromEnvErr<Inner>
Sourcepub fn from<Other>(other: FromEnvErr<Other>) -> Selfwhere
Inner: From<Other>,
pub fn from<Other>(other: FromEnvErr<Other>) -> Selfwhere
Inner: From<Other>,
Create a new error from another error type.
Sourcepub fn map<New>(self, f: impl FnOnce(Inner) -> New) -> FromEnvErr<New>
pub fn map<New>(self, f: impl FnOnce(Inner) -> New) -> FromEnvErr<New>
Map the error to another type. This is useful for converting the error type to a different type, while keeping the other error information intact.
Sourcepub const fn parse_error(err: Inner) -> Self
pub const fn parse_error(err: Inner) -> Self
Error while parsing.