Struct EnvItemInfo

pub struct EnvItemInfo {
    pub var: &'static str,
    pub description: &'static str,
    pub optional: bool,
}
Details about an environment variable. This is used to generate documentation for the environment variables and by the FromEnv trait to check if necessary environment variables are present.

Fields

var: &'static str

The environment variable name.

description: &'static str

A description of the environment variable function in the CFG.

optional: bool

Whether the environment variable is optional or not.