Module otlp

OpenTelemetry utilities.

Structs

OtelConfig
Otel configuration. This struct is intended to be loaded from the env vars
OtelGuard
Drop guard for the Otel provider. This will shutdown the provider when dropped, and generally should be held for the lifetime of the main function.
OtlpParseError
Otlp parse error.