#[non_exhaustive]pub struct OtelConfig {
pub endpoint: Url,
pub level: Level,
pub timeout: Duration,
pub environment: String,
}Expand description
Otel configuration. This struct is intended to be loaded from the env vars
The env vars it checks are:
OTEL_EXPORTER_OTLP_ENDPOINT- optional. The endpoint to send traces to, should be some valid URL. If not specified, thenOtelConfig::loadwill returnNone.- OTEL_LEVEL - optional. Specifies the minimum
tracing::Levelto export. Defaults totracing::Level::DEBUG. - OTEL_TIMEOUT - optional. Specifies the timeout for the exporter in milliseconds. Defaults to 1000ms, which is equivalent to 1 second.
- OTEL_ENVIRONMENT_NAME - optional. Value for the
deployment.environment. nameresource key according to the OTEL conventions.
Fields (Non-exhaustive)
This struct is marked as non-exhaustive
Non-exhaustive structs could have additional fields added in future. Therefore, non-exhaustive structs cannot be constructed in external crates using the traditional
Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.endpoint: UrlThe endpoint to send traces to, should be some valid HTTP endpoint for OTLP.
level: LevelDefaults to DEBUG.
timeout: DurationDefaults to 1 second. Specified in Milliseconds.
environment: StringOTEL convenition deployment.environment.name
Implementations
Sourceimpl OtelConfig
impl OtelConfig
Sourcepub fn load() -> Option<Self>
pub fn load() -> Option<Self>
Load from env vars.
The env vars it checks are:
OTEL_EXPORTER_OTLP_ENDPOINT- optional. The endpoint to send traces to. If missing or unparsable, this function will returnNone, and OTLP exporting will be disabled.OTEL_LEVEL- optional. Specifies the minimumtracing::Levelto export. Defaults totracing::Level::DEBUG.OTEL_TIMEOUT- optional. Specifies the timeout for the exporter in milliseconds. Defaults to 1000ms, which is equivalent to 1 second.OTEL_ENVIRONMENT_NAME- optional. Value for thedeployment.environment.nameresource key according to the OTEL conventions. Defaults to"unknown".