Module provider

Module provider 

Available on crate feature alloy only.
Alloy Provider configuration and instantiation

Structs

ProviderConfig
Configuration for an Alloy provider, sourced from an environment variable.
PubSubConfig
Configuration for an Alloy provider, used to create a client, enforces pubsub availability (WS or IPC connection).

Enums

ProviderConnectError
Errors when connecting a provider