pub enum LocalOrAwsConfigEnvError {
KeyInfo(<Cow<'static, str> as FromEnvVar>::Error),
ChainId(<Option<u64> as FromEnvVar>::Error),
}Available on crate feature
alloy only.Expand description
Generated error type for FromEnv for
LocalOrAwsConfig
. This error type is used to represent errors that occur when trying to create an instance of the struct from environment variables.
Variants
KeyInfo(<Cow<'static, str> as FromEnvVar>::Error)
Error for KeyInfo
ChainId(<Option<u64> as FromEnvVar>::Error)
Error for ChainId