Slot calculator for determining the current slot and timepoint within a slot.
Structs
- Slot
Calculator - A slot calculator, which can calculate slot numbers, windows, and offsets for a given chain.
Enums
- Slot
Calculator EnvError - Generated error type for
FromEnvfor SlotCalculator . This error type is used to represent errors that occur when trying to create an instance of the struct from environment variables.