pub enum SlotCalculatorEnvError {
StartTimestamp(<u64 as FromEnvVar>::Error),
SlotOffset(<u64 as FromEnvVar>::Error),
SlotDuration(<u64 as FromEnvVar>::Error),
}Expand description
Generated error type for FromEnv for
SlotCalculator
. This error type is used to represent errors that occur when trying to create an instance of the struct from environment variables.
Variants
StartTimestamp(<u64 as FromEnvVar>::Error)
Error for StartTimestamp
SlotOffset(<u64 as FromEnvVar>::Error)
Error for SlotOffset
SlotDuration(<u64 as FromEnvVar>::Error)
Error for SlotDuration