Struct SlotCalculator

pub struct SlotCalculator { /* private fields */ }
A slot calculator, which can calculate the slot number for a given timestamp.

Implementations

Source

impl SlotCalculator

pub const fn new( start_timestamp: u64, slot_offset: u64, slot_duration: u64, ) -> Self

Creates a new slot calculator.

pub const fn holesky() -> Self

Creates a new slot calculator for Holesky.

pub const fn pecorino_host() -> Self

Creates a new slot calculator for Pecorino host network.

pub const fn mainnet() -> Self

Creates a new slot calculator for Ethereum mainnet.

pub const fn calculate_slot(&self, timestamp: u64) -> u64

Calculates the slot for a given timestamp. This only works for timestamps that are GEQ to the chain’s start_timestamp.

pub const fn calculate_timepoint_within_slot(&self, timestamp: u64) -> u64

Calculates how many seconds into the block window for a given timestamp.

pub const fn calculate_slot_window(&self, slot_number: u64) -> (u64, u64)

Calculates the start and end timestamps for a given slot

pub fn current_slot(&self) -> u64

The current slot number.

pub fn current_timepoint_within_slot(&self) -> u64

The current number of seconds into the block window.

pub const fn start_timestamp(&self) -> u64

The timestamp of the first PoS block in the chain.

pub const fn slot_offset(&self) -> u64

The slot number of the first PoS block in the chain.

pub const fn slot_duration(&self) -> u64

The slot duration, usually 12 seconds.