FromEnv, FromEnvVar traits and related utilities.
Structs
- EnvItem
Info - Details about an environment variable. This is used to generate
documentation for the environment variables and by the
FromEnvtrait to check if necessary environment variables are present.
Enums
- From
EnvErr - Error type for loading from the environment. See the
FromEnvtrait for more information.
Traits
- FromEnv
- Trait for loading from the environment.
- From
EnvVar - Trait for loading primitives from the environment. These are simple types
that should correspond to a single environment variable. It has been
implemented for common integer types,
String,url::Url,tracing::Level, andstd::time::Duration.
Functions
- parse_
env_ if_ present - Convenience function for parsing a value from the environment, if present and non-empty.
Derive Macros
- FromEnv
- The
derive(FromEnv)macro.