pub struct EnvItemInfo {
pub var: &'static str,
pub description: &'static str,
pub optional: bool,
}Expand description
Details about an environment variable. This is used to generate
documentation for the environment variables and by the FromEnv trait to
check if necessary environment variables are present.
Fields
var: &'static strThe environment variable name.
description: &'static strA description of the environment variable function in the CFG.
optional: boolWhether the environment variable is optional or not.