OpenTelemetry utilities.
Structs
- Otel
Config - Otel configuration. This struct is intended to be loaded from the env vars
- Otel
Guard - Drop guard for the Otel provider. This will shutdown the provider when
dropped, and generally should be held for the lifetime of the
mainfunction. - Otlp
Parse Error - Otlp parse error.