Struct OtelConfig

#[non_exhaustive]
pub struct OtelConfig { pub endpoint: Url, pub level: Level, pub timeout: Duration, pub environment: String, }
Otel configuration. This struct is intended to be loaded from the env vars

The env vars it checks are:

  • OTEL_EXPORTER_OTLP_ENDPOINT - optional. The endpoint to send traces to, should be some valid URL. If not specified, then OtelConfig::load will return None.
  • OTEL_LEVEL - optional. Specifies the minimum tracing::Level to export. Defaults to tracing::Level::DEBUG.
  • OTEL_TIMEOUT - optional. Specifies the timeout for the exporter in milliseconds. Defaults to 1000ms, which is equivalent to 1 second.
  • OTEL_ENVIRONMENT_NAME - optional. Value for the deployment.environment. name resource key according to the OTEL conventions.

Fields (Non-exhaustive)

This struct is marked as non-exhaustive
Non-exhaustive structs could have additional fields added in future. Therefore, non-exhaustive structs cannot be constructed in external crates using the traditional Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.
endpoint: Url

The endpoint to send traces to, should be some valid HTTP endpoint for OTLP.

level: Level

Defaults to DEBUG.

timeout: Duration

Defaults to 1 second. Specified in Milliseconds.

environment: String

OTEL convenition deployment.environment.name

Implementations

Source

impl OtelConfig

pub fn load() -> Option<Self>

Load from env vars.

The env vars it checks are:

  • OTEL_EXPORTER_OTLP_ENDPOINT - optional. The endpoint to send traces to. If missing or unparsable, this function will return None, and OTLP exporting will be disabled.
  • OTEL_LEVEL - optional. Specifies the minimum tracing::Level to export. Defaults to tracing::Level::DEBUG.
  • OTEL_TIMEOUT - optional. Specifies the timeout for the exporter in milliseconds. Defaults to 1000ms, which is equivalent to 1 second.
  • OTEL_ENVIRONMENT_NAME - optional. Value for the deployment.environment.name resource key according to the OTEL conventions. Defaults to "unknown".
pub fn provider(&self) -> OtelGuard

Instantiate a new Otel provider, and start relevant tasks. Return a guard that will shut down the provider when dropped.