Available on crate feature
alloy only.Expand description
Alloy Provider configuration and instantiation
Structs
- Provider
Config - Configuration for an Alloy provider, sourced from an environment variable.
- PubSub
Config - Configuration for an Alloy provider, used to create a client, enforces pubsub availability (WS or IPC connection).